#-------------------------------------------------------#
#     add_c_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :                            #
#     Two of 32-bit complex numbers are added to each   #
#     other, producing a 32-bit result.			#
#   * Design Assumption and Constraints :		#
#     When overflow, return 2147483647                  #
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r0+) = X_R_L ( Lower of real X )            #
#          (r0+) = X_R_H ( Higher of real X )           #
#          (r0+) = X_I_L ( Lower of imaginary X )       #
#          (r0+) = X_I_H ( Higher of imaginary X )      #
#          (r1+) = Y_R_L ( Lower of real Y )            #
#          (r1+) = Y_R_H ( Higher of real Y )           #
#          (r1+) = Y_I_L ( Lower of imaginary Y )       #
#          (r1+) = Y_I_H ( Higher of imaginary Y )      #
#   * Return values :                                   #
#          (r4+) = Lower word of real result         	#
#          (r4+) = Higher word of real result        	#
#          (r4+) = Lower word of imaginary result    	#
#          (r4+) = Higher word of imaginary result	#
#   * Altered Registers					#
#          Acc, r0, r1, r3, r4, r7, stack, st           #
#   * Computation time :                                #
#          maximum 49 cycles                            #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-------------------------------------------------------
    # initial part ( User definable )
	ldi	r0, 10			# pointer of X
	ldi	r1, 20			# pointer of Y
	ldi	r4, 10			# pointer of output_L
    # main part
    add_c_dp():
    # Real part calculation
	call	always, @add_r_dp()
    # Imaginary part calculation
	call	always, @add_r_dp()
	ld 	pc, stack
#------------------------------------------------------
    # included sub routine
    add_r_dp():
      	mod	f, resop
      	ld	a, (r0+)     		# Lower word of	X
        add     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
        ld      a, (r0+)                # Higher word of X
      	bra	l=0, @add_r_dp_higher
      	addi	1                       # add carry
    add_r_dp_higher:
        mod     f, setop                
        add	a, (r1+)                # Higher word of Y
        bra	ov=1, @add_r_dp_overf_sat
    add_r_dp_save:
        ld	(r4+), stack         	# save lower word of result
        bra	always, @add_r_dp_ret
    # Overflow or Underflow Saturation !
    add_r_dp_overf_sat:
        bra	n=0, @add_r_dp_res_pos
        ld 	(r4+), -   		# saturation value : 0x7fffffff
        bra	always, @add_r_dp_ret
    add_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    add_r_dp_ret:
        ld	(r4+), a                # higher word of output
        ld	pc, stack

