#-------------------------------------------------------#
#     add_r_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :                            #
#     Two 32-bit numbers are added to each other,	#
#     producing a 32-bit result. 			#
#   * Design Assumption and Constraints :		#
#     When overflow, return 2147483647                  #
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r0+) = X_L ( Lower of X )                	#
#          (r0+) = X_H ( Higher of X )               	#
#          (r1+) = Y_L ( Lower of Y )                	#
#          (r1+) = Y_H ( Higher of Y )               	#
#   * Return values :                                   #
#          (r4+) = Lower word of result              	#
#          (r4+) = Higher word of result             	#
#   * Altered Registers	:				#
#          Acc, r0, r1, r3, r4, r7, stack, st           #
#   * Computation time :                                #
#          maximum 20 cycles                            #
#                                                       #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-------------------------------------------------------
    # initial part ( User definable )
    	ldi	r0, 10			# pointer of X
    	ldi	r1, 20                  # pointer of Y
    	ldi	r4, 10			# pointer of output
    # main part
    add_r_dp():
      	mod	f, resop
      	ld	a, (r0+)   		# Lower word of	X
        add     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
        ld      a, (r0+)                # Higher word of X
      	bra	l=0, @add_r_dp_higher
      	addi	1                       # add carry
    add_r_dp_higher:
        mod     f, setop 
        add	a, (r1+)                # Higher word of Y
        bra	ov=1, @add_r_dp_overf_sat
    add_r_dp_save:
        ld	(r4+), stack       	# save lower word of result
        bra	always, @add_r_dp_ret
    # Overflow or Underflow Saturation !
    add_r_dp_overf_sat:
        bra	n=0, @add_r_dp_res_pos
        ld 	(r4+), -   		# saturation value : 0x7fffffff
        bra	always, @add_r_dp_ret
    add_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    add_r_dp_ret:
        ld	(r4+), a		# higher word of output
        ld	pc, stack
