#-------------------------------------------------------#
#     div_r_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :                            #
#     32-bit devidend is devided by 32-bit divisor	#
#     producing 16-bit quotient as a result		#
#							#
#   * Design Assumption and Constraints :		#
#     a < c ,  b < d                                    #
#   * Input values :                                    #
#	   (r0+) = divisor				#
#	   (r4+) = low word of dividend 		#
#	   (r4+) = high word of dividend 		#
#   * Return values :                                   #
#          (r6+) = Quotient of real part		#
#   * Temporary values :                                #
#	   RAM0[0] = buffer for divisor			#
#	   RAM0[1] = subc counter number                #
#	   RAM1[0] = buffer for low word dividend       #
#	   RAM1[1] = buffer for high word dividend      #
#	   RAM1[2] = sign of devided output		#
#   * Altered Registers :				#
#          Acc, r0, r3, r4, r6, r7, stack, st, x, y     #
#   * Computation time :                                #
#          maximum 270 cycles                           #
#                                                       #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#--------------------------------------------------------
    # initial part
    	ldi	r0, 03			# pointer of divisor
    	ldi     r4, 04			# pointer of dividend
    	ldi	r6, 10			# pointer of output
    div_r_dp():
        ldi	(r3|01), 0e		# subc count number
    # get sign of quotient
        ld	ext7, (r4+)		# dividend_low
        ld	a, (r4)			# dividend_high
        mod	always, abs             # to make positive denom
        ld	(r7|01), a 		# save abs. input to buffer
        ld      (r7|00), ext7
        cmp	a, (r0)                 # check division condition
        bra	n=0, @div_impossible
        mld	(r4+), (r0) 		# divisor * dividend_high
        ld	(r7|10), p   	        # sign of quotient
        ld	a, (r0+)	        # to make positive divisor
        mod	always, abs
        ld	(r3|00), a
	ldi	y, 4000         	# for divisor shift
    	ld	x, (r3|00) 		# load divisor
        ld      ext7, (r7|00)           # dividend low 16bit
        mod	always, shr
        mod	always, shl		# clear LSB 1bit
    div_sub_loop:
        ld	a, (r7|01)		# dividend higher 16bit
	cmp	a, p
        bra	n=1, @div_defficient_remainder
        sub	a, p       		# make partial remainder
        mod	always, inc             # generate quotient
        ld	(r7|01), a		# save acc
    div_defficient_remainder:          	# if divisor>remainder
    	mod	always, shl
        ld	(r7|01), a		# save acc
        ld	a, (r3|01)
        subi	1
        ld	(r3|01), a
        bra	n=1, @div_apply_sign
    	bra	always, @div_sub_loop
    div_impossible:
    	ldi	ext7, 1
    div_apply_sign:
    	ld	(r6), ext7           	# save quotient
    	ld	a, (r7|10)		# check quotient sign
    	and	a, -
    	ld	a, (r6)		      	# final low dividend = quotent
    	bra	n=0, @div_r_dp_ret
    	mod	always, neg             # apply sign
    div_r_dp_ret:
        ld	(r6+), a		# save signed quotent
        ld	pc, stack
