#-------------------------------------------------------#
#     mult_r_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :                            #
#     Two 32-bit complex numbers are multiplied to each #
#     other, producing a 32-bit complex result 		#
#                                              		#
#	(A+Bi)*(C+Di)=(AC-BD)+(BC+AD)i			#
#                                                       #
#   * Design Assumption and Constraints :		#
#     When overflow, return 2147483647                  #
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r2+) = A_L ( Lower of X Real part ) 	#
#          (r2+) = A_H ( Higher of X Real part )	#
#          (r2+) = B_L ( Lower of X Imaginary part ) 	#
#          (r2+) = B_H ( Higher of X Imaginary part ) 	#
#          (r5+) = C_L ( Lower of Y Real part )        	#
#          (r5+) = C_H ( Higher of Y Real part )      	#
#          (r5+) = D_L ( Lower of Y Imaginary part )  	#
#          (r5+) = D_H ( Higher of Y Imaginary part ) 	#
#   * Return values :                                   #
#          (r4+) = Lower word of Real result		#
#          (r4+) = Higher word of Real result           #
#          (r4+) = Lower word of Imaginary result       #
#          (r4+) = Higher word of Imaginary result      #
#   * Temporary values :                                #
#          RAM0[00] = abs of X_L buffer ( Lower of X )  #
#          RAM0[01] = abs of X_H buffer ( Higher of X ) #
#          RAM1[00] = abs of Y_L buffer ( Lower of Y )  #
#          RAM1[01] = abs of Y_H buffer ( Higher of Y ) #
#	   RAM1[10] = Sign buffer                       #
#	   RAM1[11] = temporary buffer			#
#   * Altered Registers	:				#
#          Acc, r0, r2, r3, r4, r5, r7, stack, st, 	#
#	   x, y, p                       		#
#   * Computation time :                                #
#          maximum 274 cycles                           #
#                                                       #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-------------------------------------------------------#
    # initial part ( User Definable )
	ldi	r2, 11		# pointer of A
	ldi	r5, 11		# pointer of C
        ldi     r0, 20          # pointer of input to sub_r_dp()
        ldi     r4, 20          # pointer of output of sub_r_dp()
    # for test
        ldi	a, 2020
        ld	010, a
        ldi	a, 3030
        ld	011, a
        ldi	a, dfe0
        ld	012, a
        ldi	a, cfcf
        ld	013, a
        ldi	a, 0303
        ld	110, a
        ldi	a, 0202
        ld	111, a
        ldi	a, fcfd
        ld	112, a
        ldi	a, fdfd
        ld	113, a
    # save ponter of input to sub..
    	ld	a, r0
        ld      (r3|11), a
    # main part
    mult_c_dp():
    # real part calculation ( A*C-B*D )
    	ld	a, r5
    	ld	(r3|10), a	# save pointer of C
    	call	always, @mult_r_dp()
    	call	always, @mult_r_dp()
    	ld	a, (r3|11)
    	ld      r0, a           # restore pointer of input X to sub..
    	addi	2
        ld      r1, a           # restore pointer of input Y to sub..
    	call	always, @sub_r_dp()
    # imaginary calculation ( B*C+A*D )
        ld	a, (r3|10)
        ld      r5, a           # restore pointer of C
        ld	a, (r3|11)
    	ld      r0, a           # restore pointer of input to sub..
        ld	-, (r2-)	# move poiter to B
        ld	-, (r2-)
    	call	always, @mult_r_dp()
        ld	-, (r2-)	# move poiter to A
        ld	-, (r2-)
        ld	-, (r2-)
        ld	-, (r2-)
    	call	always, @mult_r_dp()
    	ld	-, (r2+)
    	ld	-, (r2+)
    	
    	ld	a, (r3|11)
    	ld      r0, a           # restore pointer of input to sub..
    	addi	2
    	ld	r1, a
    	call	always, @add_r_dp()
        ret
#-----------------------------------------------------------#
# 		Included subroutines                        #
#-----------------------------------------------------------#
    mult_r_dp():
	ld	a, st
	ori	a, 0200		# resop, st[9]=1 (mac shift on)
	ld	st, a
    # check sign
        mld	(r5-), (r2-)	# Y_H * X_H
        ld	(r7|10), p	# sign of the result
    # get absolute value of Y
        ld	ext7, (r5+)
        ld	a, (r5+)
        mod	always, abs     # if Y<0, find -Y
        ld	(r7|01), a      # save -Y_H
        ld	(r7|00), ext7   # save -Y_L
        ld	a, (r7|00)
        mod	f, resl
        mod     always, ror     # shift right to make unsigned value
        ld	(r7|00), a      # save -Y_L >> 1
    # get absolute value of X
        ld      ext7, (r2+)
        ld      a, (r2+)
        mod     always, abs      # if X<0, find -X
        ld      (r3|01), a       # save -X_H
        ld      (r3|00), ext7    # save -X_L
        ld      a, (r3|00)
        mod	f, resl
        mod     always, ror      # shift right to make unsigned value
        ld	(r3|00), a       # save -X_L >> 1
    mult_r_dp_cal:
	mld     (r7|00), (r3|00) # acc=0, p=(Y_L>>1)*(X_L>>1)
	ld	(r7|11), p       # higher 16bit p result is initial-
                                 # -value of following mac operation
	mld 	(r7|00), (r3|01) # p=(Y_L>>1)*X_H
	ld	ext7, (r7|11)
        mpya    (r7|01), (r3|00) # p=Y_H*(X_L>>1)
    mid_sum:
        mpya    (r7|01), (r3|00) # dummy for (add a, p)
        mod	always, shl
        ld	x, a
        ldi	y, 1             # shift right 16
        andi	00
    last_mult:
    	mod	f, setop
        mpya    (r7|01), (r3|01)  # p=Y_H*X_H
        mpya    (r5+), (r2+)  	# dummy for (add a, p)
        ld	(r0+), ext7     # save Result_Low
        ld	(r0+), a	# save Result_High
        ld      a, (r7|10)        
    	and	a, -
        bra     n=0, @pos_out
        ld	-, (r0-)        # dummy for decreasing pointer
    	ld	a, (r0-)
        ld      ext7, (r0)
        mod     always, neg     # if sign = negative -> neg. operation
        ld      (r0+), ext7     # save Result_Low
        ld      (r0+), a        # save Result_High
    pos_out:
    	ld	pc, stack
#--------------------------------------------------------
    add_r_dp():
      	mod	f, resop
      	ld	a, (r0+)   		# Lower word of	X
        add     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
        ld      a, (r0+)                # Higher word of X
      	bra	l=0, @add_r_dp_higher
      	addi	1                       # add carry
    add_r_dp_higher:
        mod     f, setop
        add	a, (r1+)                # Higher word of Y
        bra	ov=1, @add_r_dp_overf_sat
    add_r_dp_save:
        ld	(r4+), stack       	# save lower word of result
        bra	always, @add_r_dp_ret
    # Overflow or Underflow Saturation !
    add_r_dp_overf_sat:
        bra	n=0, @add_r_dp_res_pos
        ld 	(r4+), -   		# saturation value : 0x7fffffff
        bra	always, @add_r_dp_ret
    add_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    add_r_dp_ret:
        ld	(r4+), a
        ld	pc, stack
        
#--------------------------------------------------------
    sub_r_dp():
      	mod	f, resop
      	ld	a, (r0+)     		# Lower word of	X
        sub     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
      	ld	a, (r0+)
      	bra	l=1, @sub_r_dp_higher
      	subi	1                       # add carry
    sub_r_dp_higher:
        mod	f, setop                # Higher word of X
        sub	a, (r1+)                # Higher word of Y
        bra	ov=1, @sub_r_dp_overf_sat
    sub_r_dp_save:
        ld	(r4+), stack            # save lower word of result
        bra	always, @sub_r_dp_ret
    # Overflow or Underflow Saturation !
    sub_r_dp_overf_sat:
        bra	n=0, @sub_r_dp_res_pos
        ld 	(r4+), -   		# saturation value : 0x7fffffff
        bra	always, @sub_r_dp_ret
    sub_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    sub_r_dp_ret:
        ld	(r4+), a    		# save higher word of result
        ld	pc, stack
                                   
