#-------------------------------------------------------#
#     mult_r_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :                            #
#     Two 32-bit number are multiplied to each other,	#
#     producing a 32-bit result 			#
#                                              		#
#        		WWWW|XXXX  ( 32-bit X )         #
#                     * YYYY|ZZZZ  ( 32-bit Y )         #
#		      -----------                       #
#                     + XXXX*ZZZZ                       #
#	         + WWWW*ZZZZ                            #
#	 	 + YYYY*XXXX                            #
#  	    + WWWW*YYYY                                 #
#           ----------------------                      #
#             AAAA|BBBB  ( 32-bit output )              #
#                                                       #
#   * Design Assumption and Constraints :		#
#     When overflow, return 2147483647                	#
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r2+) = X_L ( Lower of X )                	#
#          (r2+) = X_H ( Higher of X )              	#
#          (r5+) = Y_L ( Lower of Y )                	#
#          (r5+) = Y_H ( Higher of Y )                  #
#   * Return values :                                   #
#          (r0+) = Lower word of result                 #
#          (r0+) = Higher word of result                #
#   * Temporary values :                                #
#          RAM0[00] = abs of X_L buffer ( Lower of X )  #
#          RAM0[01] = abs of X_H buffer ( Higher of X ) #
#          RAM1[00] = abs of Y_L buffer ( Lower of Y )  #
#          RAM1[01] = abs of Y_H buffer ( Higher of Y ) #
#	   RAM1[10] = Sign buffer                       #
#   * Altered Registers	:				#
#          Acc, r0, r2, r3, r5, r7, stack, st, x, y, p  #
#   * Computation time :                                #
#          maximum 52 cycles                            #
#                                                       #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-------------------------------------------------------#
    # initial part ( User definable )
	ldi	r2, 11		# pointer of X
	ldi	r5, 11		# pointer of Y
	ldi	r0, 20		# pointer of the result
    # main part
    mult_r_dp():
	ld	a, st
	ori	a, 0200		# resop, st[9]=1 (mac shift on)
	ld	st, a
    # check sign
        mld	(r5-), (r2-)	# Y_H * X_H
        ld	(r7|10), p	# sign of the result
    # get absolute value of Y
        ld	ext7, (r5+)
        ld	a, (r5+)
        mod	always, abs     # if Y<0, find -Y
        ld	(r7|01), a      # save -Y_H
        ld	(r7|00), ext7   # save -Y_L
        ld	a, (r7|00)
        mod	f, resl
        mod     always, ror     # shift right to make unsigned value
        ld	(r7|00), a      # save -Y_L >> 1
    # get absolute value of X
        ld      ext7, (r2+)
        ld      a, (r2+)
        mod     always, abs      # if X<0, find -X
        ld      (r3|01), a       # save -X_H
        ld      (r3|00), ext7    # save -X_L
        ld      a, (r3|00)
        mod	f, resl
        mod     always, ror      # shift right to make unsigned value
        ld	(r3|00), a       # save -X_L >> 1
    mult_r_dp_cal:
	mld     (r7|00), (r3|00) # acc=0, p=(Y_L>>1)*(X_L>>1)
	ld	(r7|11), p       # higher 16bit p result is initial-
                                 # -value of following mac operation
	mld 	(r7|00), (r3|01) # p=(Y_L>>1)*X_H
	ld	ext7, (r7|11)
        mpya    (r7|01), (r3|00) # p=Y_H*(X_L>>1)
    mid_sum:
        mpya    (r7|01), (r3|00) # dummy for (add a, p)
        mod	always, shl
        ld	x, a
        ldi	y, 1             # shift right 15
        andi	00
    last_mult:
    	mod	f, setop
        mpya    (r7|01), (r3|01)  # p=Y_H*X_H
        mpya    (r5+), (r2+)    # dummy for (add a, p)
        ld	(r0+), ext7     # save Result_Low
        ld	(r0+), a        # save Result_High
        ld      a, (r7|10)        
    	and	a, -
        bra     n=0, @pos_out
        ld	-, (r0-)	# dummy for decreasing pointer
    	ld	a, (r0-)
        ld      ext7, (r0)
        mod     always, neg     # if sign = negative -> neg. operation
        ld      (r0+), ext7     # save Result_Low
        ld      (r0+), a        # save Result_High
    pos_out:
    	ld	pc, stack

