#-------------------------------------------------------#
#     sub_c_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :				#
#     Two 32-bit complex numbers are subtracted to 	#
#     each other, producing a 32-bit complex result	#
#   * Design Assumption and Constraints : 		#
#     When overflow, return 2147483647                  #
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r0+) = X_R_L ( Lower of real X )            #
#          (r0+) = X_R_H ( Higher of real X )           #
#          (r0+) = X_I_L ( Lower of imaginary X )       #
#          (r0+) = X_I_H ( Higher of imaginary X )      #
#          (r1+) = Y_R_L ( Lower of real Y )            #
#          (r1+) = Y_R_H ( Higher of real Y )           #
#          (r1+) = Y_I_L ( Lower of imaginary Y )       #
#          (r1+) = Y_I_H ( Higher of imaginary Y )      #
#   * Return values :                                   #
#          (r4+) = Lower word of real result            #
#          (r4+) = Higher word of real result           #
#          (r4+) = Lower word of imaginary result       #
#          (r4+) = Higher word of imaginary result      #
#   * Altered Registers	:				#
#          Acc, r0, r1, r3, r4, r7, stack, st           #
#   * Computation times :                               #
#          maximum 49 cycles                            #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-----------------------------------------------------
    # initial part ( User definable )
	ldi	r0, 10             	# pointer of X
	ldi	r1, 20			# pointer of Y
	ldi	r4, 10			# pointer of output
    # main part
    sub_c_dp():
    # Real part calculation
	call	always, @sub_r_dp()
    # Imaginary part calculation
	call	always, @sub_r_dp()
	ld 	pc, stack
	
#---------------------------------------
    # included sub routine
    sub_r_dp():
      	mod	f, resop
      	ld	a, (r0+)   		# Lower word of	X
        sub     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
      	ld	a, (r0+)
      	bra	l=1, @sub_r_dp_higher
      	subi	1                       # sub borrow
    sub_r_dp_higher:
        mod	f, setop                # Higher word of X
        sub	a, (r1+)                # Higher word of Y
        bra	ov=1, @sub_r_dp_overf_sat
    sub_r_dp_save:
        ld	(r4+), stack         	# save lower word of result
        bra	always, @sub_r_dp_ret
    # Overflow or Underflow Saturation !
    sub_r_dp_overf_sat:
        bra	n=0, @sub_r_dp_res_pos
        ld 	(r4+), -		# saturation value : 0x7fffffff
        bra	always, @sub_r_dp_ret
    sub_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    sub_r_dp_ret:
        ld	(r4+), a        	# save higher word of result
        ld	pc, stack

