#-------------------------------------------------------#
#     sub_r_dp.sc					#
#-------------------------------------------------------#
#   * Function Description :				#
#     Two of 32-bit numbers are subtracted to each 	#
#     other, producing a 32-bit result.			#
#     32 bit subtraction of two 32bit variables         #
#   * Design Assumption and Constraints :		#
#     When overflow, return 2147483647                  #
#  	   underflow, return -2147483648                #
#   * Input values :                                    #
#          (r0+) = X_L ( Lower of X )                   #
#          (r0+) = Y_L ( Lower of Y )                   #
#          (r0+) = X_H ( Higher of X )                  #
#          (r0+) = Y_H ( Higher of Y )                  #
#   * Return values :                                   #
#          (r4+) = Lower word of result                 #
#          (r4+) = Higher word of result                #
#   * Altered Registers	:				#
#          Acc, x, y, r3, r7, st                        #
#   * Computation time :				#
#          20 cycles                                    #
#                                                       #
#-------------------------------------------------------#
org	0000
dw	0004
dw	0004
dw	0004
dw	0004
org	0004
#-------------------------------------------------------
    # initial part ( User definable )
    	ldi	r0, 10			# pointer of X
    	ldi	r1, 20			# pointer of Y
    	ldi	r4, 10			# pointer of output
    # main part
    sub_r_dp():
      	mod	f, resop
      	ld	a, (r0+)   		# Lower word of	X
        sub     a, (r1+)                # Lower word of Y
        ld	stack, a        	# save sum of lower word
      	ld	a, (r0+)
      	bra	l=1, @sub_r_dp_higher
      	subi	1                       # sub borrow
    sub_r_dp_higher:
        mod	f, setop                # Higher word of X
        sub	a, (r1+)                # Higher word of Y
        bra	ov=1, @sub_r_dp_overf_sat
    sub_r_dp_save:
        ld	(r4+), stack            # save lower word of result
        bra	always, @sub_r_dp_ret
    # Overflow or Underflow Saturation !
    sub_r_dp_overf_sat:
        bra	n=0, @sub_r_dp_res_pos
        ld 	(r4+), -		# saturation value : 0x7fffffff
        bra	always, @sub_r_dp_ret
    sub_r_dp_res_pos:
        ldi	(r4+), 0000             # saturation value : 0x80000000
    sub_r_dp_ret:
        ld	(r4+), a                # save higher word of result
        ld	pc, stack
